# This file is part of iCLIPS.

# ICLIPS is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.

# ICLIPS is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the
# GNU General Public License for more details.

# You should have received a copy of the GNU General Public License
# along with ICLIPS. If not, see <http://www.gnu.org/licenses/>.


KEYWORDS = ('SYMBOL',
            'STRING',
            'INTEGER',
            'FLOAT',
            'crlf',
            'object',
            'deftemplate',
            'deffunction',
            'defmodule',
            'defrule',
            'defclass',
            'defglobal',
            'deffacts',
            'test',
            'and',
            'or',
            'eq',
            'neq',
            'name',
            'is-a',
            'type',
            'slot',
            'multislot',
            't ')

BUILTINS = ('!=',
            '*',
            '**',
            '+',
            '-',
            '/',
            '<',
            '<=',
            '<>',
            '=',
            '>',
            '>=',
            'abs',
            'acos',
            'acosh',
            'acot',
            'acoth',
            'acsc',
            'acsch',
            'active-duplicate-instance',
            'active-initialize-instance',
            'active-make-instance',
            'active-message-duplicate-instance',
            'active-message-modify-instance',
            'active-modify-instance',
            'agenda',
            'and',
            'any-instancep',
            'apropos',
            'asec',
            'asech',
            'asin',
            'assert',
            'cot',
            'coth',
            'create$',
            'csc',
            'csch',
            'defclass-module',
            'deffacts-module',
            'deffunction-module',
            'defgeneric-module',
            'defglobal-module',
            'definstances-module',
            'defrule-module',
            'deftemplate-module',
            'deg-grad',
            'deg-rad',
            'delayed-do-for-all-instances',
            'delete$',
            'delete-instance',
            'dependencies',
            'dependents',
            'describe-class',
            'direct-mv-delete',
            'direct-mv-insert',
            'direct-mv-replace',
            'div',
            'do-for-all-instances',
            'do-for-instance',
            'dribble-off',
            'dribble-on',
            'duplicate',
            'duplicate-instance',
            'duplicate-instance',
            'dynamic-get',
            'dynamic-put',
            'edit',
            'eq',
            'eval',
            'evenp',
            'exit',
            'exp',
            'expand$',
            'explode$',
            'facts',
            'fact-existp',
            'halt',
            'if',
            'implode$',
            'init-slots',
            'initialize-instance',
            'initialize-instance',
            'insert$',
            'instance-address',
            'instance-addressp',
            'instance-existp',
            'instance-name',
            'instance-name-to-symbol',
            'instance-namep',
            'instancep',
            'instances',
            'integer',
            'integerp',
            'length',
            'length$',
            'lexemep',
            'list-defclasses',
            'list-deffacts',
            'list-deffunctions',
            'list-defgenerics',
            'list-defglobals',
            'list-definstances',
            'list-defmessage-handlers',
            'list-defmethods',
            'list-defmodules',
            'list-defrules',
            'list-deftemplates',
            'list-focus-stack',
            'list-watch-items',
            'load',
            'load*',
            'load-facts',
            'load-instances',
            'log',
            'log10',
            'loop-for-count',
            'lowcase',
            'make-instance',
            'ppdefclass',
            'ppdeffacts',
            'ppdeffunction',
            'ppdefgeneric',
            'ppdefglobal',
            'ppdefinstances',
            'ppdefmessage-handler',
            'ppdefmethod',
            'ppdefmodule',
            'ppdefrule',
            'ppdeftemplate',
            'ppinstance',
            'preview-generic',
            'preview-send',
            'primitives-info',
            'print-region',
            'printout',
            'progn',
            'progn$',
            'put',
            'rad-deg',
            'random',
            'read',
            'readline',
            'refresh',
            'refresh-agenda',
            'release-mem',
            'remove',
            'remove-break',
            'rename',
            'replace$',
            'reset',
            'rest$',
            'restore-instances',
            'retract',
            'return',
            'round',
            'rule-complexity',
            'rules',
            'run',
            'save',
            'save-facts',
            'save-instances',
            'str-assert',
            'str-cat',
            'str-compare',
            'str-explode',
            'str-implode',
            'str-index',
            'str-length',
            'stringp',
            'sub-string',
            'subclassp',
            'subseq$',
            'subset',
            'subsetp',
            'superclassp',
            'switch',
            'sym-cat',
            'symbol-to-instance-name',
            'symbolp',
            'system',
            'tan',
            'tanh',
            'time',
            'toss',
            'type',
            'type',
            'undefclass',
            'undeffacts',
            'undeffunction',
            'undefgeneric',
            'undefglobal',
            'undefinstances',
            'undefmessage-handler',
            'undefmethod',
            'undefrule',
            'undeftemplate',
            'unmake-instance',
            'unwatch',
            'upcase',
            'watch',
            'while',
            'wordp')
